@@ -4,74 +4,80 @@
 #' 
 #' tabset, table layout, support for GIS shapefiles through library 'sf'
 #'  
-#' @param cmd utility command, see Details\cr
-#' @param js optional JavaScript function, default is NULL.\cr
+#' @param cmd utility command, see Details
+#' @param js optional JavaScript function, default is NULL.
+#' @param event optional event name for cmd='morph'.
 #' @param ... Optional parameters for the command \cr
 #'      for \emph{sf.series} - see \href{https://echarts.apache.org/en/option.html#series-scatter.type}{points}, \href{https://echarts.apache.org/en/option.html#series-lines.type}{polylines}, polygons(itemStyle).\cr
 #'      for \emph{tabset} parameters should be in format \emph{name1=chart1, name2=chart2}, see example\cr
 #' @details 
 #' **cmd = 'sf.series'**\cr
-#' \verb{ }Build _leaflet_ or \href{https://echarts.apache.org/en/option.html#geo.map}{geo} map series from shapefiles.\cr
-#' \verb{ }Supported types: POINT, MULTIPOINT, LINESTRING, MULTILINESTRING, POLYGON, MULTIPOLYGON \cr
-#' \verb{ }Coordinate system is _leaflet_(default), _geo_ or _cartesian3D_ (for POINT(xyz))\cr
-#' \verb{ }Limitations:\cr 
+#' \verb{  }Build _leaflet_ or \href{https://echarts.apache.org/en/option.html#geo.map}{geo} map series from shapefiles.\cr
+#' \verb{  }Supported types: POINT, MULTIPOINT, LINESTRING, MULTILINESTRING, POLYGON, MULTIPOLYGON \cr
+#' \verb{  }Coordinate system is _leaflet_(default), _geo_ or _cartesian3D_ (for POINT(xyz))\cr
+#' \verb{  }Limitations:\cr 
 #' \verb{     }polygons can have only their name in tooltip,  \cr
 #' \verb{     }assumes Geodetic CRS is WGS 84, for conversion use \link[sf]{st_transform} with _crs=4326_.\cr
-#' \verb{ }Parameters:\cr 
-#' \verb{   }df - value from \link[sf]{st_read}\cr
-#' \verb{   }nid - optional column name for name-id used in tooltips\cr
-#' \verb{   }cs - optional _coordinateSystem_ value, default 'leaflet'\cr
-#' \verb{   }verbose - optional, print shapefile item names in console\cr
-#' \verb{ }Returns a list of chart series\cr
+#' \verb{  }Parameters:\cr 
+#' \verb{     }df - value from \link[sf]{st_read}\cr
+#' \verb{     }nid - optional column name for name-id used in tooltips\cr
+#' \verb{     }cs - optional _coordinateSystem_ value, default 'leaflet'\cr
+#' \verb{     }verbose - optional, print shapefile item names in console\cr
+#' \verb{  }Returns a list of chart series\cr
 #' **cmd = 'sf.bbox'**\cr
-#' \verb{ }Returns JavaScript code to position a map inside a bounding box from \link[sf]{st_bbox}, for leaflet only.\cr
+#' \verb{  }Returns JavaScript code to position a map inside a bounding box from \link[sf]{st_bbox}, for leaflet only.\cr
 #' **cmd = 'sf.unzip'**\cr
-#' \verb{ }Unzips a remote file and returns local file name of the unzipped .shp file\cr
-#' \verb{   }url - URL of remote zipped shapefile\cr
-#' \verb{   }shp - optional name of .shp file inside ZIP file if multiple exist. Do not add file extension. \cr
+#' \verb{  }Unzips a remote file and returns local file name of the unzipped .shp file\cr
+#' \verb{     }url - URL of remote zipped shapefile\cr
+#' \verb{     }shp - optional name of .shp file inside ZIP file if multiple exist. Do not add file extension. \cr
+#' \verb{  }Returns full name of unzipped .shp file, or error string starting with 'ERROR'\cr
 #' **cmd = 'geojson'** \cr
-#' \verb{ }Custom series list from geoJson objects\cr
-#' \verb{   }geojson - object from \link[jsonlite]{fromJSON}\cr
-#' \verb{   }cs - optional _coordinateSystem_ value, default 'leaflet'\cr
-#' \verb{   }ppfill - optional fill color like '#F00', OR NULL for no-fill, for all Points and Polygons\cr
-#' \verb{   }nid - optional feature property for item name used in tooltips\cr
-#' \verb{   }... - optional custom series attributes like _itemStyle_\cr
-#' \verb{ }Can display also geoJson _feature properties_: color; lwidth, ldash (lines); ppfill, radius (points)\cr
+#' \verb{  }Custom series list from geoJson objects\cr
+#' \verb{     }geojson - object from \link[jsonlite]{fromJSON}\cr
+#' \verb{     }cs - optional _coordinateSystem_ value, default 'leaflet'\cr
+#' \verb{     }ppfill - optional fill color like '#F00', OR NULL for no-fill, for all Points and Polygons\cr
+#' \verb{     }nid - optional feature property for item name used in tooltips\cr
+#' \verb{     }... - optional custom series attributes like _itemStyle_\cr
+#' \verb{  }Can display also geoJson _feature properties_: color; lwidth, ldash (lines); ppfill, radius (points)\cr
 #' **cmd = 'layout'** \cr
-#' \verb{ }Multiple charts in table-like rows/columns format\cr
-#' \verb{   }... - List of charts\cr
-#' \verb{   }title - optional title for the set, rows= Number of rows, cols= Number of columns\cr
-#' \verb{ }Returns a container \link[htmltools]{div} in rmarkdown, otherwise \link[htmltools]{browsable}.\cr
-#' \verb{ }For 3-4 charts one would use multiple series within a \href{https://echarts.apache.org/en/option.html#grid}{grid}. \cr
-#' \verb{ }For greater number of charts _ec.util(cmd='layout')_ comes in handy\cr
+#' \verb{  }Multiple charts in table-like rows/columns format\cr
+#' \verb{     }... - List of charts\cr
+#' \verb{     }title - optional title for the entire set\cr
+#' \verb{     }rows - optional number of rows\cr 
+#' \verb{     }cols - optional number of columns\cr
+#' \verb{  }Returns a container \link[htmltools]{div} in rmarkdown, otherwise \link[htmltools]{browsable}.\cr
+#' \verb{  }For 3-4 charts one would use multiple series within a \href{https://echarts.apache.org/en/option.html#grid}{grid}. \cr
+#' \verb{  }For greater number of charts _ec.util(cmd='layout')_ comes in handy\cr
 #' **cmd = 'tabset'** \cr
-#' \verb{   }... - a list name/chart pairs like \emph{n1=chart1, n2=chart2}, each tab may contain a chart.\cr
-#' \verb{   }tabStyle - tab style string, see default \emph{tabStyle} variable in the code\cr
-#' \verb{ }Returns A) \link[htmltools]{tagList} of tabs when in a pipe without '...' params, see example\cr
-#' \verb{ }Returns B) \link[htmltools]{browsable} when '...' params are provided by user\cr
+#' \verb{     }... - a list name/chart pairs like \emph{n1=chart1, n2=chart2}, each tab may contain a chart.\cr
+#' \verb{     }tabStyle - tab style string, see default \emph{tabStyle} variable in the code\cr
+#' \verb{  }Returns A) \link[htmltools]{tagList} of tabs when in a pipe without '...' params, see example\cr
+#' \verb{  }Returns B) \link[htmltools]{browsable} when '...' params are provided by user\cr
+#' \verb{  }Please note that sometimes those tabsets do not merge well inside advanced web pages.\cr
 #' **cmd = 'button'** \cr
-#' \verb{ }UI button to execute a JS function,\cr
-#' \verb{   }text - the button label\cr
-#' \verb{   }js - the JS function string\cr
-#' \verb{   }... - optional parameters for the \href{https://echarts.apache.org/en/option.html#graphic.elements-rect.type}{rect} element\cr
-#' \verb{ }Returns a graphic.elements-\href{https://echarts.apache.org/en/option.html#graphic.elements-rect.type}{rect} element.\cr
+#' \verb{  }UI button to execute a JS function,\cr
+#' \verb{     }text - the button label\cr
+#' \verb{     }js - the JS function string\cr
+#' \verb{     }... - optional parameters for the \href{https://echarts.apache.org/en/option.html#graphic.elements-rect.type}{rect} element\cr
+#' \verb{  }Returns a graphic.elements-\href{https://echarts.apache.org/en/option.html#graphic.elements-rect.type}{rect} element.\cr
 #' **cmd = 'morph'** \cr
-#' \verb{   }... - a list of charts or chart options\cr
-#' \verb{   }js - optional JS function for switching charts. Default function is on \emph{mouseover}. Disable with FALSE.\cr
-#' \verb{ }Returns a chart with ability to morph into other charts\cr
+#' \verb{     }... - a list of charts or chart option lists\cr
+#' \verb{     }event - name of event for switching charts. Default is \emph{click}.\cr
+#' \verb{  }Returns a chart with ability to morph into other charts\cr
 #' **cmd = 'fullscreen'** \cr
-#' \verb{ }A toolbox feature to toggle fullscreen on/off. Works in a browser, not in RStudio.\cr
+#' \verb{  }A toolbox feature to toggle fullscreen on/off. Works in a browser, not in RStudio.\cr
 #' **cmd = 'rescale'** \cr
-#' \verb{   }v - input vector of numeric values to rescale\cr
-#' \verb{   }t - target range c(min,max), numeric vector of two\cr
+#' \verb{     }v - input vector of numeric values to rescale\cr
+#' \verb{     }t - target range c(min,max), numeric vector of two\cr
 #' **cmd = 'level'** \cr
-#' \verb{ }Calculate vertical levels for timeline \emph{line} charts, returns a numeric vector\cr
-#' \verb{   }df - data.frame with _from_ and _to_ columns\cr
-#' \verb{   }from - name of 'from' column\cr
-#' \verb{   }to - name of 'to' column\cr
+#' \verb{  }Calculate vertical levels for timeline \emph{line} charts, returns a numeric vector\cr
+#' \verb{     }df - data.frame with _from_ and _to_ columns\cr
+#' \verb{     }from - name of 'from' column\cr
+#' \verb{     }to - name of 'to' column\cr
 #' 
-#' @examples 
-#' if (interactive()) {  # comm.out: Fedora errors about some 'browser'
+#' @examples
+#' library(dplyr)
+#' if (interactive()) {  # comm.out: Cran Fedora errors about some 'browser'
 #'   library(sf)
 #'   fname <- system.file("shape/nc.shp", package="sf")
 #'   nc <- as.data.frame(st_read(fname))
@@ -82,34 +88,59 @@
 #'   )
 #' 
 #'   htmltools::browsable(
-#'     lapply(iris |> dplyr::group_by(Species) |> dplyr::group_split(), 
+#'     lapply(iris |> group_by(Species) |> group_split(), 
 #'            function(x) {
 #'      x |> ec.init(ctype= 'scatter', title= list(text= unique(x$Species)))
 #'            }) |> 
 #'     ec.util(cmd= 'tabset')
 #'   )
 #' 
-#'   p1 <- cars |> ec.init(grid= list(top= 20))  # move chart up
-#'   p2 <- mtcars |> ec.init()
-#'   ec.util(cmd= 'tabset', cars= p1, mtcars= p2, width= 333, height= 333)
+#'   p1 <- cars |> ec.init(grid= list(top=26), height=333)  # move chart up
+#'   p2 <- mtcars |> arrange(mpg) |> ec.init(height=333, ctype='line')
+#'   ec.util(cmd= 'tabset', cars= p1, mtcars= p2)
+#' 
+#' 
+#'   cars |> ec.init(
+#'     graphic = list(
+#'       ec.util(cmd='button', text='see type', right='center', top=20,
+#'         js="function(a) {op=ec_option(echwid); alert(op.series[0].type);}")
+#'     )
+#'   )
 #' 
 #'   lapply(list('dark','macarons','gray','jazz','dark-mushroom'),
-#'                 \(x) cars |> ec.init() |> ec.theme(x) ) |>
+#'     function(x) cars |> ec.init(grid= list(bottom=0)) |> ec.theme(x) ) |>
 #'   ec.util(cmd='layout', cols= 2, title= 'my layout')
-#'   
-#'   setd <- \(type) {
-#' 	   mtcars |> group_by(cyl) |> 
-#' 	 ec.init(ctype= type,
-#' 		  title= list(subtext= 'mouseover points to morph'),
-#' 		  xAxis= list(scale= TRUE))
-#'   }
-#'   oscatter <- setd('scatter')
-#'   obar <- setd('bar')
-#'   ec.util(cmd='morph', oscatter, obar)
 #' }
+#' 
+#' colors <- c("blue","red","green")
+#' cyls <- as.character(sort(unique(mtcars$cyl)))
+#' sers <- lapply(mtcars |> group_by(cyl) |> group_split(), \(x) {
+#'   cyl <- as.character(unique(x$cyl))
+#'   list(type='scatter', id=cyl, dataGroupId=cyl, 
+#'        data= ec.data(x |> select(mpg,hp)),
+#'        universalTransition= TRUE)
+#' })
+#' oscatter <- list(
+#'   title= list(subtext='click points to morph'), 
+#'   color= colors, tooltip= list(show=TRUE),
+#'   xAxis= list(scale=TRUE, name='mpg'), yAxis= list(scale=TRUE, name='hp'),
+#'   series= sers
+#' )
+#' opie <- list(
+#'   title= list(text= 'Average hp'), 
+#'   color= colors, tooltip= list(show=TRUE),
+#'   series= list(list(
+#'     type= 'pie', label= list(show=TRUE), colorBy= 'data',
+#'     data= ec.data(mtcars |> group_by(cyl) |> summarize(value= mean(hp)) |>
+#'        mutate(groupId= as.character(cyl), name= as.character(cyl)),'names'),
+#'     universalTransition= list(enabled=TRUE, seriesKey= cyls)
+#'   ))
+#' )
+#' ec.util(cmd='morph', oscatter, opie) 
+#'   
 #' @importFrom utils unzip
 #' @export
-ec.util <- function( ..., cmd='sf.series', js=NULL) {
+ec.util <- function( ..., cmd='sf.series', js=NULL, event='click') {
   
   opts <- list(...)
   
@@ -120,7 +151,7 @@ ec.util <- function( ..., cmd='sf.series', js=NULL) {
       j <- j + 1
       val <- NULL
       if (!is.null(dflts)) val <- dflts[[j]]
-      tmp <- unname(unlist(opts[n]))
+      tmp <- unlist(opts[n], use.names=FALSE)
       if (!is.null(tmp)) {
         val <- tmp
         opts[n] <<- NULL
@@ -163,8 +194,9 @@ ec.util <- function( ..., cmd='sf.series', js=NULL) {
               tt <- c(paste(rep('%@', length(flds)), collapse='<br>'), flds)
             }
             pnts <- ec.data(dff, 'names')
+            styp <- ifelse(endsWith(cs, '3D'), 'scatter3D', 'scatter')
             sers <- list( c(
-              list(type= 'scatter', coordinateSystem= cs, data= pnts), opts))
+              list(type= styp, coordinateSystem= cs, data= pnts), opts))
             if (!is.null(tt)) 
               sers[[1]]$tooltip= list(formatter= do.call("ec.clmn", as.list(tt)))
           },
@@ -226,7 +258,7 @@ ec.util <- function( ..., cmd='sf.series', js=NULL) {
           if (len==1) len <- length(x$data[[1]])  #multiline
           len
         })))
-        cat('\n series:',cnt,'coords:',recs,'\n')
+        cat('\n series:',cnt,'records:',recs,'\n')
         sers
       }
       
@@ -250,17 +282,23 @@ ec.util <- function( ..., cmd='sf.series', js=NULL) {
     },
     'sf.unzip'= {
       stopifnot('ec.util: expecting url of zipped shapefile'= !is.null(opts$url))
-      destfile <- tempfile('shapefile')
-      download.file(opts$url, destfile, mode='wb') #, method='curl')
-      # get name only, use as folder name to unzip to
-      fldr <- sub(pattern = "(.*)\\..*$", replacement = "\\1", basename(opts$url))
-      unzip(destfile, exdir=fldr)  # new folder under getwd()
-      # find name
-      pat <- ifelse (is.null(opts$shp), '*.shp', paste0(opts$shp,'.shp'))
-      tmp <- list.files(path= fldr, pattern= pat)
-      if (length(tmp)==0) 
-        stop(paste('ec.util:',pat,'file not found in folder',fldr), call. = FALSE)
-      out <- paste0(getwd(),'/',fldr,'/',tmp[1])
+      #stopifnot('ec.util: invalid zip url'= .valid.url(opts$url))
+      if (!.valid.url(opts$url)) { out <- 'ERROR invalid zip url' }
+      else {
+        destfile <- tempfile('shapefile')
+        download.file(opts$url, destfile, mode='wb') #, method='curl')
+        # get name only, use as folder name to unzip to
+        fldr <- sub(pattern = "(.*)\\..*$", replacement = "\\1", basename(opts$url))
+        unzip(destfile, exdir=fldr)  # new folder under getwd()
+        # find name
+        pat <- ifelse (is.null(opts$shp), '*.shp', paste0(opts$shp,'.shp'))
+        tmp <- list.files(path= fldr, pattern= pat)
+        if (length(tmp)==0) 
+          #  stop(paste('ec.util:',pat,'file not found in folder',fldr), call. = FALSE)
+          out <- 'ERROR unzipped file not found'
+        else
+          out <- paste0(getwd(),'/',fldr,'/',tmp[1])
+      }
     },
     
     'geojson'= {
@@ -289,8 +327,7 @@ ec.util <- function( ..., cmd='sf.series', js=NULL) {
     
     'tabset'= {
       tabStyle <- NULL   # CRAN check fix
-      do.opties(c('tabStyle'), 
-                list("<style>
+      do.opties(c('tabStyle'), list("<style>
 /*	CSS for the main interaction */
 .tabset > input[type='radio'] {
  position: absolute;
@@ -339,7 +376,7 @@ body {
  margin-bottom: -1px;
 }
 .tab-panel {
- padding: 10px 0;
+ padding: 10px 0; width: 100%;
  border-top: 1px solid #ccc;
 }
 body { padding: 10px; }
@@ -396,6 +433,7 @@ body { padding: 10px; }
       
       title <- NULL   # CRAN check fix
       do.opties(c('rows','cols','title'))
+      stopifnot("ec.util: list of charts is missing"= length(opts)>0)
       lplots <- length(opts[[1]])
       if (is.null(rows) & !is.null(cols)) rows <- ceiling(lplots/cols)
       if (!is.null(rows) & is.null(cols)) cols <- ceiling(lplots/rows)
@@ -435,8 +473,7 @@ body { padding: 10px; }
         else oo
       })
       # series types should be different for morph options
-      clickHandler <- htmlwidgets::JS("
-    function(event) {
+      morfHandler <- htmlwidgets::JS("function(event) {
         opt= this.getOption();
         keep= opt.morph;
         for(i=0; i<keep.length; i++) {
@@ -445,18 +482,16 @@ body { padding: 10px; }
        		  optcurr= Object.assign({}, keep[next]);
        		  break;
     	    }
-    	 };
-    	 if (!optcurr) return;
-    	 optcurr.morph= keep;
-    	 this.setOption(optcurr, true);
-    }")
+    	  };
+    	  if (!optcurr) return;
+    	  optcurr.morph= keep;
+    	  this.setOption(optcurr, true);
+      }")
       out <- ec.init(preset=FALSE, js=js)
       out$x$opts <- opts[[1]]
       out$x$opts$morph <- opts
-      if (is.null(js))
-        out$x$on <- list(list(
-          event= 'click', handler= clickHandler
-        ))
+      #if (is.null(event)) event <- 'click'
+      out$x$on <- list(list(event= event, handler= morfHandler))
       out    
     },
     
@@ -567,13 +602,13 @@ body { padding: 10px; }
 #'   
 #' @details 
 #' `format='boxplot'` requires the first two _df_ columns as: \cr
-#' \verb{   }column for the non-computational categorical axis\cr
-#' \verb{   }column with (numeric) data to compute the five boxplot values\cr
+#' \verb{     }column for the non-computational categorical axis\cr
+#' \verb{     }column with (numeric) data to compute the five boxplot values\cr
 #'  Additional grouping is supported on a column after the second. Groups will show in the legend, if enabled.\cr
 #'  Returns a `list(dataset, series, xAxis, yAxis)` to set params in [ec.init]. 
 #'  Make sure there is enough data for computation, 4+ values per boxplot.\cr
 #' `format='treeTT'` expects data.frame _df_ columns _pathString,value,(optional itemStyle)_ for \link[data.tree]{FromDataFrameTable}.\cr
-#'  It will add column 'pct' with value percentage for each node. See Details.
+#'  It will add column 'pct' with value percentage for each node. See example below.
 #' @seealso some live \href{https://rpubs.com/echarty/data-models}{code samples}
 #' 
 #' @examples
@@ -590,7 +625,7 @@ body { padding: 10px; }
 #'   series= list(list(
 #'     type= 'tree', orient= 'TB', roam= TRUE, initialTreeDepth= -1,
 #'     data= ec.data(hc, format='dendrogram'),
-#'     # layout= 'radial', symbolSize= ec.clmn(scale= 0.33),
+#'     layout= 'radial', # symbolSize= ec.clmn(scale= 0.33),
 #'     ## exclude added labels like 'pXX', leaving only the originals
 #'     label= list(formatter= htmlwidgets::JS(
 #'       "function(n) { out= /p\\d+/.test(n.name) ? '' : n.name; return out;}"))
@@ -667,7 +702,7 @@ ec.data <- function(df, format='dataset', header=FALSE, ...) {
   
   if (format=='treePC') {
     # for sunburst,tree,treemap
-    if (!all(unlist(unname(lapply(as.list(df[,1:3]), class))) == 
+    if (!all(unlist(lapply(as.list(df[,1:3]), class), use.names=FALSE) == 
              c('character','character','numeric')) )
       stop('ec.data: df columns need to be in order (parents, children, value), only value is numeric', call. = FALSE)
     
@@ -728,7 +763,7 @@ ec.data <- function(df, format='dataset', header=FALSE, ...) {
       datset <- c(list(colnames(df)), datset)
   } 
   else if (format=='values' || isTRUE(format)) {
-    datset <- lapply(tmp, \(x) list(value=unlist(unname(x))))
+    datset <- lapply(tmp, \(x) list(value=unlist(x, use.names=FALSE)))
   } 
   else if (format=='boxplot') {
     args <- list(...)
@@ -845,11 +880,11 @@ ec.data <- function(df, format='dataset', header=FALSE, ...) {
     if (jitter>0) {
   		tmp <- df |> arrange(.data[[colas]]) |> 
   		        group_by(.data[[colas]]) |> group_split()
-  		mcyl <- lapply(tmp, \(x) unname(unlist(x[[colb5]])))
+  		mcyl <- lapply(tmp, \(x) unlist(x[[colb5]], use.names=FALSE))
   		names(mcyl) <- sort(unique(df[[colas]]))
   		i <- 0.5
   		serj <- lapply(names(mcyl), \(nn) {  
-  			yy <- unname(unlist(mcyl[nn]))
+  			yy <- unlist(mcyl[nn], use.names=FALSE)
   			xx <- jitter(rep(i, length(yy)), amount= jitter)
   			out <- list(type= 'scatter', ...)
   			if (!'name' %in% names(args)) out$name <- nn
@@ -911,11 +946,11 @@ ec.data <- function(df, format='dataset', header=FALSE, ...) {
 #' Helper function to display/format data column(s) by index or name
 #' 
 #' @param col A single column index(number) or column name(quoted string), \cr
-#'    or a \link[base]{sprintf} string template for multiple indexes.\cr
+#'    or a \link[base]{sprintf} string template for multiple columns.\cr
 #'    NULL(default) for charts with single values like tree, pie.\cr
 #'    'json' to display tooltip with all available values to choose from.\cr 
 #'    'log' to write all values in the JS console (F12) for debugging.\cr
-#'    Can contain JS function starting with _'function('_ (or format _'(x) => {}'_).\cr
+#'    Can contain JS function starting with _'function('_ or _'(x) => \{'_.\cr
 #' @param ... Comma separated column indexes or names, only when \emph{col} is \emph{sprintf}. This allows formatting of multiple columns, as for a tooltip.\cr
 #' @param scale A positive number, multiplier for numeric columns. When scale is 0, all numeric values are rounded.
 #' @return A JavaScript code string (usually a function) marked as executable, see \link[htmlwidgets]{JS}.
@@ -932,23 +967,33 @@ ec.data <- function(df, format='dataset', header=FALSE, ...) {
 #' * \emph{%LR@} rounded number in locale format, like '12,345'.\cr
 #' * \emph{%R@} rounded number, like '12345'.\cr
 #' * \emph{%R2@} rounded number, two digits after decimal point.\cr
-#' * \emph{%M@} marker in serie's color.\cr
-#' Notice that tooltip _formatter_ will work for _trigger='item'_, but not for _trigger='axis'_ when there are multiple value sets.
+#' * \emph{%M@} marker in series' color.\cr
+#' For _trigger='axis'_ (multiple series) one can use decimal column indexes.\cr
+#' See definition above and example below.
 #' 
 #' @examples
+#' library(dplyr)
 #' tmp <- data.frame(Species = as.vector(unique(iris$Species)),
 #'                   emoji = c('A','B','C'))
-#' df <- iris |> dplyr::inner_join(tmp)      # add 6th column emoji
-#' df |> dplyr::group_by(Species) |> ec.init(
+#' df <- iris |> inner_join(tmp)      # add 6th column emoji
+#' df |> group_by(Species) |> ec.init(
 #'   series.param= list(label= list(show= TRUE, formatter= ec.clmn('emoji'))),
 #'   tooltip= list(formatter=
 #'     # with sprintf template + multiple column indexes
 #'     ec.clmn('%M@ species <b>%@</b><br>s.len <b>%@</b><br>s.wid <b>%@</b>', 5,1,2))
 #' )
 #' 
+#' # tooltip decimal indexes work with full data sets (no missing/partial data)
+#' ChickWeight |> mutate(Chick=as.numeric(Chick)) |> filter(Chick>47) |> group_by(Chick) |>
+#' ec.init(
+#'   tooltip= list(trigger='axis', 
+#'                 formatter= ec.clmn("48: %@<br>49: %@<br>50: %@", 1.1, 2.1, 3.1)),
+#'   xAxis= list(type='category'), legend= list(formatter= 'Ch.{name}'),
+#'   series.param= list(type='line', encode= list(x='Time', y='weight')),
+#' )
 #' @export
 ec.clmn <- function(col=NULL, ..., scale=1) {
-  if (is.null(scale)) scale=1
+  if (is.null(scale)) scale <- 1
   if (scale==1) scl <- 'return c;'
   else {
     if (scale==0) scl <- 'return Math.round(c);'
@@ -977,6 +1022,7 @@ return template.replace(/%@|%L@|%LR@|%R@|%R2@|%M@/g, (m) => {
   if (m=='%R2@') return Number(vals[j++]).toFixed(2);
   if (m=='%M@') return x.marker;
 }); };"
+	
 	if (length(args)==0) {  # col is solitary name
 		args <- col; col <- '%@'   # replace
 	}
@@ -1088,13 +1134,15 @@ ec.paxis <- function(dfwt=NULL, cols=NULL, minmax=TRUE, ...) {
   } 
   else {
     stopifnot('ec.paxis: dfwt has to be class echarty'= inherits(dfwt, 'echarty'))
-    coln <- unlist(dfwt$x$opts$dataset[[1]]$source[1])
+    #coln <- unlist(dfwt$x$opts$dataset[[1]]$source[1])
+    coln <- unlist(dfwt$x$opts$dataset[[1]]$dimensions)
     cfilter <- 1:length(coln)
     if (length(dfwt$x$opts$dataset) > 1) {
       grnm <- dfwt$x$opts$dataset[[2]]$transform$config$dimension
       cfilter <- cfilter[!cfilter==match(grnm, coln)]
     }
-    idf <- as.data.frame(t(do.call(cbind, dfwt$x$opts$dataset[[1]]$source[(-1)])))
+    #idf <- as.data.frame(t(do.call(cbind, dfwt$x$opts$dataset[[1]]$source[(-1)])))
+    idf <- as.data.frame(t(do.call(cbind, dfwt$x$opts$dataset[[1]]$source)))
     colnames(idf) <- coln
     idf <- as.data.frame(apply(idf, 2, unlist, simplify=FALSE))
   }
@@ -1172,9 +1220,9 @@ ec.theme <- function (wt, name='custom', code= NULL)
 #' 
 #' @param wt An \code{echarty} widget as returned by [ec.init]
 #' @param target type of resulting value: \cr
-#' \verb{   }'opts' - the htmlwidget _options_ as JSON (default)\cr
-#' \verb{   }'full' - the _entire_ htmlwidget as JSON\cr
-#' \verb{   }'data' - info about chart's embedded data (char vector)
+#' \verb{     }'opts' - the htmlwidget _options_ as JSON (default)\cr
+#' \verb{     }'full' - the _entire_ htmlwidget as JSON\cr
+#' \verb{     }'data' - info about chart's embedded data (char vector)
 #' @param ... Additional attributes to pass to \link[jsonlite]{toJSON}\cr
 #' 'file' - optional file name to save to when target='full'\cr
 #' @return A JSON string, except when \code{target} is 'data' - then
@@ -1251,10 +1299,10 @@ ec.inspect <- function(wt, target='opts', ...) {
 #' Convert JSON string or file to chart
 #' 
 #' @param txt Could be one of the following:\cr
-#' \verb{   }class _url_, like \code{url('https://serv.us/cars.txt')}\cr
-#' \verb{   }class _file_, like \code{file('c:/temp/cars.txt','rb')}\cr
-#' \verb{   }class _json_, like \code{ec.inspect(p)}, for options or full\cr
-#' \verb{   }class _character_, JSON string with options only, see example below\cr
+#' \verb{     }class _url_, like \code{url('https://serv.us/cars.txt')}\cr
+#' \verb{     }class _file_, like \code{file('c:/temp/cars.txt','rb')}\cr
+#' \verb{     }class _json_, like \code{ec.inspect(p)}, for options or full\cr
+#' \verb{     }class _character_, JSON string with options only, see example below\cr
 #' @param ... Any attributes to pass to internal [ec.init] when _txt_ is options only
 #' @return An _echarty_ widget.
 #' 
@@ -1271,6 +1319,7 @@ ec.inspect <- function(wt, target='opts', ...) {
 #' # cars |> ec.init() |> ec.inspect(target='full', file=outFile)
 #' # ec.fromJson(file(outFile, 'rb'))
 #' # ec.fromJson(url('http://localhost/echarty/cars.json'))
+#' 
 #' ec.fromJson('https://helgasoft.github.io/echarty/test/pfull.json')
 #' @export
 ec.fromJson <- function(txt, ...) {
