@@ -33,14 +33,13 @@ test_that("registerMap", {
 
 test_that("tl.series, timeline options, groupBy", {  # also in test-presets
   p <- Orange |> dplyr::group_by(age) |> ec.init(
-    timeline= list(autoPlay=TRUE),
+    timeline= list(autoPlay=TRUE), title= list(text='age %@ days'),
     series.param= list(type='bar', encode=list(x='Tree', y='circumference'))
-  ) |> ec.upd({
-    options <- lapply(seq_along(options), 
-      \(i) { 
-        options[[i]]$title$text <- paste('age',timeline$data[[i]],'days'); 
-        options[[i]] })
-  })
+  ) # |> ec.upd({
+  #   options <- lapply(seq_along(options), \(i) { 
+  #       options[[i]]$title$text <- paste('age',timeline$data[[i]],'days'); 
+  #       options[[i]] })
+  # })
   expect_equal(p$x$opts$options[[5]]$title$text, "age 1231 days")
   expect_equal(p$x$opts$options[[5]]$series[[1]]$datasetIndex, 5)
   expect_equal(p$x$opts$options[[7]]$series[[1]]$encode$x, "Tree")
@@ -56,10 +55,17 @@ test_that("tl.series, timeline options, groupBy", {  # also in test-presets
   p <- dat |> group_by(x1) |> ec.init(
     tl.series= list(encode= list(x= 'x3', y= 'x5'), groupBy='x2',
                     symbolSize= ec.clmn(4, scale=30)),
-    legend= list(s=T)
+    legend= list(show=T)
   )
   expect_equal(p$x$opts$options[[4]]$series[[2]]$name, 'B')
   expect_true(p$x$opts$dataset[[9]]$transform$config$and[[2]]$dimension=='x2')
+  
+  p <- data.frame(name=c('Brazil','Australia'), value=c(111,222)) |> group_by(name) |>
+  ec.init(load= 'world', title= list(text='map %@'), visualMap=list(show=T), 
+        timeline= list(show=T), series.param= list(type='map') )
+  expect_equal(p$x$opts$options[[2]]$title$text, "map Brazil")
+  expect_equal(p$x$opts$visualMap$min, 111)
+
 })
 
 test_that("leaflet with ec.clmn and timeline", {
@@ -67,12 +73,11 @@ test_that("leaflet with ec.clmn and timeline", {
   tmp <- quakes |> dplyr::relocate('long') |>  # set order to lon,lat
       dplyr::mutate(size= exp(mag)/20) |> head(100)  # add accented size
   p <- tmp |> ec.init(load= 'leaflet',
-                      tooltip = list(formatter=ec.clmn('magnitude %@', 'mag')),
-    series.param= list(
-      symbolSize = ec.clmn(6, scale=3)
-      )
+                      tooltip = list(formatter= ec.clmn('magnitude %@', 'mag')),
+    series.param= list(datasetIndex= 1, symbolSize= ec.clmn(6, scale=3) )
   )
   expect_equal(p$x$opts$leaflet$zoom, 6)
+  expect_equal(p$x$opts$series[[1]]$datasetIndex, 0)  # decremented
   expect_s3_class(p$x$opts$tooltip$formatter, 'JS_EVAL')
 
   p <- tmp |> group_by(stations) |> ec.init(load='leaflet', 
@@ -150,42 +155,68 @@ test_that("ec.data treeTK", {
   expect_equal(p$x$opts$series[[1]]$data[[1]]$children[[2]]$pct, 32.3)
 })
 
-test_that("load 3D surface", {
-  #if (interactive()) {  # first time will load echarts-gl.js in source folder 'js'
+test_that("load 3D surface", {  # volcano is a lot of data ==slow
+  if (interactive()) {  # first time will load echarts-gl.js in source folder 'js'
   data <- list()
   for(y in 1:dim(volcano)[2]) for(x in 1:dim(volcano)[1])
     data <- append(data, list(c(x, y, volcano[x,y])))
-  p <- ec.init(load= '3D', series= list(list(type= 'surface',	data= data)) )
+  p <- ec.init(load= '3D', series.param= list(type= 'surface',	data= data) )
   
   expect_equal(length(p$x$opts$series[[1]]$data), 5307)
-})
+}})
 
-test_that("3D globe", {
-  p <- ec.init(load='3D',
+test_that("3D globe & autoload 3D", {
+  p <- ec.init( #load='3D',  # test autoload
     globe= list(viewControl= list(autoRotate= FALSE)),
-    series= list(
-      list(type= 'scatter3D',
+    series.param= list(type= 'scatter3D',
         data= list(c(32,-117,11), c(2,44,22)) ,
         symbolSize= 40, itemStyle= list(color= 'red')
-    ))
+    )
   )
+  lif <- paste0(system.file('js', package='echarty'), '/echarts-gl.min.js')
+  expect_true(file.exists(lif))
   expect_equal(p$x$opts$series[[1]]$coordinateSystem, 'globe')
+  
+  p <- ec.init(load='world', geo3D= list(map= 'world', roam=T),
+    series.param= list(type= 'scatter3D', data=list(c(115, 22, 10), c(-116, 32, -11)))
+  ) 
+  expect_equal(p$x$opts$series[[1]]$coordinateSystem, 'geo3D')
+})
+
+test_that("radar and polar", {  # for coverage
+  p <- ec.init(
+  	radar= list(indicator= lapply(LETTERS[1:5], \(x) { list(name=x)} ) ),
+  	series= list(list(type='radar', radarIndex=1,
+  	                  data= list(c(10,22,5,9,11), c(12,18,15,15,7))))
+  )
+  expect_equal(p$x$opts$series[[1]]$radarIndex, 0)
+  
+  p <- data.frame(x = 1:10, y = seq(1, 20, by = 2)) |>
+  ec.init(polar= list(show=T), series.param= list(type='line', polarIndex=1))
+  expect_equal(p$x$opts$series[[1]]$polarIndex, 0)
+  
 })
 
 test_that("calendar", {
+
   df <- data.frame(d= seq(as.Date("2023-01-01"), by="day", length.out=360), v=runif(360,1,100))
-  p <- df |> ec.init(
+  p <- df |> ec.init( 
     visualMap= list(show= FALSE, min= 0, max= 100), 
     calendar = list(range= c('2023-01','2023-04')),
-    series = list(list(type = 'scatter'))
+    series.param= list(type= 'scatter', name='scat', symbolSize=11, calendarIndex=1)
   )
   expect_equal(p$x$opts$series[[1]]$coordinateSystem, "calendar")
+  expect_equal(p$x$opts$series[[1]]$calendarIndex, 0)
 })
 
 test_that("ec.plugjs", {
   p <- ec.init() |> ec.plugjs(
     'https://raw.githubusercontent.com/apache/echarts/master/test/data/map/js/china-contour.js')
   expect_equal(p$dependencies[[1]]$name, "china-contour.js")
+  
+  # .valid.url exits gracefully
+  p <- ec.init() |> ec.plugjs('http://does.not.exist.com')
+  expect_true(startsWith(p$x$opts$title$text, 'ERROR'))
 })
 
 test_that("Shiny commands", {
@@ -223,6 +254,7 @@ test_that(".merlis", {
   aa = list(x= list(type= "1st.is.named.list"), geoIndex= 0)
   p <- echarty:::.merlis(aa, list(val= 13))
   expect_equal(p$val, 13)
+  expect_equal(echarty:::.valid.url('http://does.not.exist.com'), FALSE)
 })
 
 test_that('autoset axis type', {
@@ -242,13 +274,14 @@ test_that('stops are working in echarty.R', {
   expect_silent(ec.init(mtcars |> group_by(gear), tl.series= list(type='map'))) # no name/value, can use encode
   expect_silent(ec.init(df |> group_by(y), series.param= list(type='bar')))
   expect_silent(ec.init(df |> group_by(y), series.param= list(type='bar'), 
-                        timeline= list(s=T)))
+                        timeline= list(show=T)))
   # expect_error(cars |> group_by(speed) |> ec.init()) # 3 cols min
   # expect_error(ec.init(data.frame(name='n',value=1) |> group_by(name), 
   #     tl.series= list(type='bar')))  # 3 cols min
   # expect_silent(ec.init(data.frame(name='n',value=1) |> group_by(name), 
   #     tl.series= list(type='map')))  # 2 cols exception for map
   expect_error(ec.init(mtcars |> group_by(gear), tl.series= list(encode= list(x=1, y=2),groupBy='zzz'))) # groupBy
+  expect_error(ec.util(cmd='layout'))
   expect_error(ecr.band(cars))
   tmp <- cars; tmp <- tmp |> rename(lower=speed, upper=dist)
   expect_error(ecr.band(tmp, lower='lower', upper='upper')) # no first col
@@ -264,5 +297,6 @@ test_that('stops are working in echarty.R', {
   #expect_silent(ec.init(load='liquid'))   # Debian throws warnings in CRAN check
   #expect_silent(ec.init(load='gmodular'))
   #expect_silent(ec.init(load='wordcloud'))
+  expect_error(mtcars |> group_by(cyl) |> ec.init(ctype='parallel'))
   
 })
